@@ -94,7 +94,7 @@ class Manhole(threading.Thread):
         if os.path.exists(name):
             os.unlink(name)
         sock.bind(name)
-        sock.listen(0)
+        sock.listen(5)
         cry("Manhole UDS path: "+name)
         return sock, pid
 
@@ -126,6 +126,7 @@ class ManholeConnection(threading.Thread):
         self.sigmask = sigmask
 
     def run(self):
+        cry('Started ManholeConnection thread. Checking credentials ...')
         if signalfd and self.sigmask:
             signalfd.sigprocmask(signalfd.SIG_BLOCK, self.sigmask)
         pthread_setname_np(self.ident, "Manhole ----")
@@ -222,6 +223,8 @@ def _handle_oneshot(_signum, _frame):
         # we don't want to let any exception out, it might make the application missbehave
         cry("Manhole oneshot connection failed:")
         cry(traceback.format_exc())
+    finally:
+        _remove_manhole_uds()
 
 def _remove_manhole_uds():
     name = "/tmp/manhole-%s" % os.getpid()
